import java.awt.Color;

public final class Planet implements ICollidable
{
	public static final double GRAVITY_CONSTANT = 6.67E-11;
	public static final double DISTANCE_SCALE   = .8E13;
	public static final float  MAX_GRAV_DIST    = 300;

	private float  Mass;
	private float  Diameter;
	private Vector Position;
	private Color  PColor;

	public Planet (float Mass, float Diameter, Vector Position, Color PColor)
	{
		this.Mass     = Mass;
		this.Diameter = Diameter;
		this.Position = Position;
		this.PColor   = PColor;
	}

	public float GetMass ()
	{
		return Mass;
	}

	public float GetDiameter ()
	{
		return Diameter;
	}

	public Vector GetPosVector ()
	{
		return Position;
	}

	public Color GetColor ()
	{
		return PColor;
	}

	public Vector GetGravityAcceleration (ICollidable CObject)
	{
		float  Radius  = (float)Vector.Subtract(CObject.GetPosVector(), Position).GetLength();
		Vector PAcc    = new Vector(0, 0);

		if (Radius <= MAX_GRAV_DIST)
		{
			float  GravityEffect;
			Vector GravAcc;

			GravityEffect = (float)((GRAVITY_CONSTANT * Mass)
			                         / (Radius * DISTANCE_SCALE));
			GravAcc       = Vector.Normal(CObject.GetPosVector(), Position).Unit();

			GravAcc.Multiply(GravityEffect);

			PAcc = GravAcc;
		}

		return PAcc;
	}

	public IMPlanet GetIMPlanet()
	{
		return new IMPlanet(Mass, Diameter, Position.GetIMVector());
	}
}
