import java.applet.*;
import java.awt.*;
import java.util.Random;
import java.util.ArrayList;

public final class SpaceGraphical extends Applet implements Runnable
{
	private Thread           Animator;
	private SpaceManager     SM;

	private Image            RenderImage;
	private Graphics         RenderGraphics;

	private Team[]           Teams;

	public void init()
	{
		Animator       = new Thread(this);
		SM             = new SpaceManager();

		RenderImage    = createImage(Constants.SCREEN_X_SIZE, Constants.SCREEN_Y_SIZE);
		RenderGraphics = RenderImage.getGraphics();

		RenderGraphics.setFont(new Font("Arial", Font.PLAIN, 9));

		resize(Constants.SCREEN_X_SIZE, Constants.SCREEN_Y_SIZE);
		setBackground(Color.BLACK);
	}

	public void start()
	{
		Random RandGen = new Random(System.currentTimeMillis());

		System.out.println("Starting up...");

		/* TEAM CREATION */
		Teams = new Team[2];

		for (int i = 0; i < Teams.length; i++)
		{
			Teams[i] = new Team("Team #" + i, new Color(Color.HSBtoRGB(RandGen.nextFloat(),
			                                       ClampRGB(.5f + .5f * RandGen.nextFloat()),
			                                       ClampRGB(.5f + .5f * RandGen.nextFloat()))));
		}
		/* END TEAM CREATION */

		/* PLANET CREATION */
		ArrayList<Planet> PlanetList = SM.GetPlanetList();
		PlanetList.ensureCapacity(8);
		
		for (int p = 0; p < 8; p++)
		{
			PlanetList.add(new Planet(RandGen.nextFloat() * 2E23f, (RandGen.nextFloat() * 50 + 50),
			                        new Vector ((float)(RandGen.nextDouble() *
		                                                Constants.PLAY_FIELD_X_SIZE),
		                                        (float)(RandGen.nextDouble() *
		                                                Constants.PLAY_FIELD_Y_SIZE)),
		                            new Color(Color.HSBtoRGB(RandGen.nextFloat(),
			                                       ClampRGB(.5f + .5f * RandGen.nextFloat()),
			                                       ClampRGB(.5f + .5f * RandGen.nextFloat())))));
		}
		/* END PLANET CREATION */

		/* SHIP CREATION */
		ArrayList<Ship> ShipList = SM.GetShipList();
		ShipList.ensureCapacity(2000);

		for (int i = 0; i < 1000; i++)
		{
			Vector Pos = new Vector(0, 0);

			Pos.SetX(Constants.PLAY_FIELD_X_SIZE * RandGen.nextFloat());
			Pos.SetY(Constants.PLAY_FIELD_Y_SIZE * RandGen.nextFloat());

			for (Planet CurrPlanet : SM.GetPlanetList())
			{
				Vector PlanetShipDelta = Vector.Normal(CurrPlanet.GetPosVector(), Pos);
				
				if (PlanetShipDelta.GetLength() <= CurrPlanet.GetDiameter())
					Pos.Add(Vector.Multiply(PlanetShipDelta, 2));
			}
			
			ShipList.add(new Ship("Test Ship #" + i, Pos, new Vector(0, 0), new Vector(0, 0),
			                    (Math.PI * RandGen.nextDouble()), 2500, 150, Teams[0],
			                    new DeansAI(), 2));
		}

		for (int i = 0; i < 1000; i++)
		{
			Vector Pos = new Vector(0, 0);

			Pos.SetX(Constants.PLAY_FIELD_X_SIZE * RandGen.nextFloat());
			Pos.SetY(Constants.PLAY_FIELD_Y_SIZE * RandGen.nextFloat());

			for (Planet CurrPlanet : SM.GetPlanetList())
			{
				Vector PlanetShipDelta = Vector.Normal(CurrPlanet.GetPosVector(), Pos);
				
				if (PlanetShipDelta.GetLength() <= CurrPlanet.GetDiameter())
					Pos.Add(Vector.Multiply(PlanetShipDelta, 2));
			}

			ShipList.add(new Ship("Test Ship #" + i, Pos, new Vector(0, 0), new Vector(0, 0),
			                    (Math.PI * RandGen.nextDouble()), 2500, 150, Teams[1],
			                    new RandomAI(), 2));
		}
		/* END SHIP CREATION */

		System.gc();

		if (ShipList.size() < 1000)
			SM.PrintPlayingShips(false);

		SM.UpdateIMShipArray();
		SM.UpdateIMBulletArray();

		DrawPlayingField(RenderGraphics);
		repaint();

		System.out.println("Loaded, staring animation thread...");

		Animator.setName("Animation Thread");
		Animator.setPriority(Thread.MAX_PRIORITY);
		Animator.start();
	}

	public void update(Graphics g)
	{
		g.drawImage(RenderImage, 0, 0, null);
	}

	public void paint(Graphics g)
	{
		update(g);
	}

	public void run()
	{
		long StartTime = System.currentTimeMillis();
	
		while (!(SM.IsGameOver()))
		{
			SM.DoGameLoop();

			DrawPlayingField(RenderGraphics);

			repaint();

			try
			{
			   Thread.sleep(20);
			}
			catch (InterruptedException e)
			{
			   return;
			}
		}

		String GameOverString = "Game Over!";

		RenderGraphics.setColor(Color.WHITE);
		RenderGraphics.setFont(new Font("Arial", Font.PLAIN, 30));
		RenderGraphics.drawString("Game Over!",
					  (int)((Constants.SCREEN_X_SIZE -
					   RenderGraphics.getFontMetrics().stringWidth(GameOverString)) / 2),
					  (int)((Constants.SCREEN_Y_SIZE -
					   RenderGraphics.getFontMetrics().getHeight()) / 2));

		repaint();		

		System.out.println (GameOverString);
		SM.PrintPlayingShips(true);
		
		int GameTime = (int)((System.currentTimeMillis() - StartTime) / 1000);
		int Minutes  = (GameTime / 60);
		int Seconds  = (GameTime % 60);
		
		System.out.println("  > Total Frames in Game: " + SM.GetElapsedFrames());
		System.out.println("  > Total Game Time: " + Minutes + " minutes and " +
						   Seconds + " seconds.");
	}

	public void DrawPlayingField (Graphics g)
	{
		int     TotalShips     = SM.GetShipList().size();
		float[] ShipColorArray = new float[3];

		g.setColor(Color.BLACK);
		g.fillRect(0, 0, Constants.SCREEN_X_SIZE, Constants.SCREEN_Y_SIZE);

		for (Ship CurrShip : SM.GetShipList())
		{
			Color ShipColor = CurrShip.GetTeam().GetColor();

			float ColScale;

			ColScale = (1 - ((float)CurrShip.GetTimeToFire()
			                      / Constants.SHIP_FRAMES_BETWEEN_FIRES));
			ColScale = (ColScale * .75f) + .25f;

			ShipColorArray = ShipColor.getRGBColorComponents(ShipColorArray);

			g.setColor(new Color(ClampRGB(ColScale * ShipColorArray[0]),
			                     ClampRGB(ColScale * ShipColorArray[1]),
			                     ClampRGB(ColScale * ShipColorArray[2])));

			g.fillOval((int)(Constants.SCALE_FACTOR_X *
			                 (CurrShip.GetPosVector().GetX() - Constants.SHIP_RADIUS)),
			           (int)(Constants.SCALE_FACTOR_Y *
					         (CurrShip.GetPosVector().GetY() - Constants.SHIP_RADIUS)),
			           (int)(Constants.SCALE_FACTOR_X * Constants.SHIP_DIAMETER),
					   (int)(Constants.SCALE_FACTOR_Y *Constants.SHIP_DIAMETER));

			g.setColor(ShipColor);

			if (TotalShips < 100)
			{
				g.drawString(CurrShip.GetName(),
				             (int)(Constants.SCALE_FACTOR_X *
							  (CurrShip.GetPosVector().GetX() -
				              (g.getFontMetrics().stringWidth(CurrShip.GetName()) >> 1))),
				             (int)(Constants.SCALE_FACTOR_Y *
							  (CurrShip.GetPosVector().GetY() +
				              Constants.SHIP_DIAMETER)) + 5);

				StringBuilder ShipString = new StringBuilder("(");
				ShipString.append(CurrShip.GetTeam().GetName());
				ShipString.append(", ID");
				ShipString.append(CurrShip.GetShipID());
				ShipString.append(", H");
				ShipString.append(CurrShip.GetHealth());
				ShipString.append(")");
			
				g.drawString(ShipString.toString(),
				             (int)(Constants.SCALE_FACTOR_X * (CurrShip.GetPosVector().GetX() -
				              (g.getFontMetrics().stringWidth(ShipString.toString()) >> 1))),
				             (int)(Constants.SCALE_FACTOR_Y * (CurrShip.GetPosVector().GetY() +
					          Constants.SHIP_DIAMETER)) + 13);

				StringBuilder AIString = new StringBuilder(CurrShip.GetAI().GetName());
				AIString.append(" (");
				AIString.append(CurrShip.GetAI().GetInfo());
				AIString.append(")");
				
				g.drawString(AIString.toString(),
				             (int)(Constants.SCALE_FACTOR_X *
							  (CurrShip.GetPosVector().GetX() -
				              (g.getFontMetrics().stringWidth(AIString.toString()) >> 1))),
				             (int)(Constants.SCALE_FACTOR_Y *
							  (CurrShip.GetPosVector().GetY() +
					          Constants.SHIP_DIAMETER) + 21));
			}

			g.setColor(Color.BLACK);

			g.drawLine((int)(Constants.SCALE_FACTOR_X * CurrShip.GetPosVector().GetX()),
			           (int)(Constants.SCALE_FACTOR_Y * CurrShip.GetPosVector().GetY()),
			           (int)(Constants.SCALE_FACTOR_X * (CurrShip.GetPosVector().GetX() +
			                 Math.cos(CurrShip.GetHeading()) * Constants.SHIP_RADIUS)),
			           (int)(Constants.SCALE_FACTOR_Y * (CurrShip.GetPosVector().GetY() +
			                 Math.sin(CurrShip.GetHeading()) * Constants.SHIP_RADIUS)));

			//CurrShip.GetAI().DrawDebug(CurrShip.GetIMShipExt(), g);
		}

		g.setColor(Color.WHITE);

		for (Bullet CurrBullet : SM.GetBulletList())
		{
			g.fillOval((int)(Constants.SCALE_FACTOR_X * (CurrBullet.GetPosVector().GetX() -
			                                   Constants.BULLET_RADIUS)),
			           (int)(Constants.SCALE_FACTOR_Y * (CurrBullet.GetPosVector().GetY() -
			                                   Constants.BULLET_RADIUS)),
			           (int)(Constants.SCALE_FACTOR_X * Constants.BULLET_DIAMETER),
					   (int)(Constants.SCALE_FACTOR_Y * Constants.BULLET_DIAMETER));
		}

		for (Planet CurrPlanet : SM.GetPlanetList())
		{
			int    PlanetDiameter = (int)CurrPlanet.GetDiameter();
			int    PlanetRadius   = (int)(PlanetDiameter / 2);

			g.setColor(CurrPlanet.GetColor());

			g.drawOval((int)(Constants.SCALE_FACTOR_X *
			                (CurrPlanet.GetPosVector().GetX() - PlanetRadius)),
			           (int)(Constants.SCALE_FACTOR_Y *
					        (CurrPlanet.GetPosVector().GetY() - PlanetRadius)),
			           (int)(Constants.SCALE_FACTOR_X * PlanetDiameter),
			           (int)(Constants.SCALE_FACTOR_Y * PlanetDiameter));
		}
	}

	private static float ClampRGB(float RGBVal)
	{
		if (RGBVal > 1f)
			return 1;
		else
			return RGBVal;
	}
}
