public final class Vector implements IVector
{
	private float   X, Y;

	public Vector(float X, float Y)
	{
		this.X = X;
		this.Y = Y;
	}

	public float GetX()
	{
		return X;
	}

	public float GetY()
	{
		return Y;
	}

	public void SetX(float NewX)
	{
		X = NewX;
	}

	public void SetY(float NewY)
	{
		Y = NewY;
	}
	
	public void Set(IVector V)
	{
		X = V.GetX();
		Y = V.GetY();
	}

	public void Set(float NewX, float NewY)
	{
		X = NewX;
		Y = NewY;
	}

	public double GetLength()
	{
		return Math.sqrt((X * X) + (Y * Y));
	}

	public Vector Add(IVector V)
	{
		X += V.GetX();
		Y += V.GetY();

		return this;
	}

	public Vector Add(float AddX, float AddY)
	{
		X += AddX;
		Y += AddY;

		return this;
	}
	
	public Vector Subtract(IVector V)
	{
		X -= V.GetX();
		Y -= V.GetY();

		return this;
	}

	public Vector Subtract(float SubX, float SubY)
	{
		X -= SubX;
		Y -= SubY;

		return this;
	}

	public Vector Multiply(float Scaler)
	{
		X *= Scaler;
		Y *= Scaler;

		return this;
	}

	public Vector Multiply(float MulX, float MulY)
	{
		X *= MulX;
		Y *= MulY;

		return this;
	}

	public Vector Divide(float Scaler)
	{
		X /= Scaler;
		Y /= Scaler;

		return this;
	}

	public Vector Divide(float DivX, float DivY)
	{
		X /= DivX;
		Y /= DivY;

		return this;
	}

	public Vector Unit()
	{
		double Length = Math.sqrt((X * X) + (Y * Y));

		X /= Length;
		Y /= Length;
	
		return this;
	}
	
	public IMVector GetIMVector()
	{
		return new IMVector(X, Y);
	}

	public static Vector Normal(IVector V1, IVector V2)
	{
		return new Vector(V2.GetX() - V1.GetX(), V2.GetY() - V1.GetY());
	}

	public static Vector Tangent(IVector V)
	{
		return new Vector(-V.GetY(), V.GetX());
	}
	
	public static float DotProduct(IVector V1, IVector V2)
	{
		if ((V1.GetLength() == 0) || (V2.GetLength() == 0))
			return 0;
		else
			return (float)(((V1.GetX() * V2.GetX()) + (V1.GetY() * V2.GetY()))
		                  / (V1.GetLength() * V2.GetLength()));
	}

	public static float CrossProduct(IVector V1, IVector V2)
	{

		return ((V1.GetX() * V2.GetY()) - (V1.GetY() * V2.GetX()));
	}

	public static double GetAngle(IVector V1, IVector V2)
	{
		return Math.atan2((V2.GetY() - V1.GetY()), (V2.GetX() - V1.GetX()));
	}

	public static Vector Add(IVector V1, IVector V2)
	{
		return new Vector(V1.GetX() + V2.GetX(), V1.GetY() + V2.GetY());
	}

	public static Vector Subtract(IVector V1, IVector V2)
	{
		return new Vector(V2.GetX() - V1.GetX(), V2.GetY() - V1.GetY());
	}

	public static Vector Multiply(IVector V1, IVector V2)
	{
		return new Vector(V1.GetX() * V2.GetX(), V1.GetY() * V2.GetY());
	}

	public static Vector Multiply(IVector V1, float Scaler)
	{
		return new Vector(V1.GetX() * Scaler, V1.GetY() * Scaler);
	}

	public static Vector Divide(IVector V1, IVector V2)
	{
		return new Vector(V1.GetX() / V2.GetX(), V1.GetY() / V2.GetY());
	}

	public static Vector Divide(IVector V1, float Scaler)
	{
		return new Vector(V1.GetX() / Scaler, V1.GetY() / Scaler);
	}

	public static Vector Unit(IVector V)
	{
		double Length = V.GetLength();
	
		return new Vector((float)(V.GetX() / Length), (float)(V.GetY() / Length));
	}
}
